#!/usr/bin/env python

_author = 'Vinny Valdez (vvaldez@redhat.com)'
_major_version = '1.0'
_minor_version = '42'

"""
UI for dellConfigCluster.py

This application collects data from users,
and calls threads of the backend to perform
actions on remote systems, including adding
them to a cluster, creating a GFS shared
storage, and configuring basic applications.

This is only intended for use on initial deployment
of 2-16 node clusters.  Management should be
done by systme-config-cluster or Conga.

This software is licensed under the terms of the GPL.
"""

import subprocess, time, re, getopt, sys, webbrowser
import dellConfigCluster 

try:
	import pygtk
	pygtk.require("2.0")
except:
  	pass
try:
	import gtk
	import gtk.glade
except:
	sys.exit(1)

class DellConfigClusterGui(gtk.Assistant):

	def usage():
		print "Usage: dell-config-cluster-gui [-h,--help,-D,--debug,-d,--demo]"

	try:
		opts, args = getopt.getopt(sys.argv[1:], "hDdfa", ["help","debug","demo","fast","apps"])
	except getopt.GetoptError, err:
		# print help information and exit:
		print str(err)
		usage()
		sys.exit(2)
	DEBUG = False
	DEMO = False
	FASTDEMO = False
	APPS_ONLY = False
	LAB_EQLOGIC = False
	LAB_MD3000 = False
	LAB_MD3000i = False
	
	for i, j in opts:
		if i in ("-D","--debug"):
			print "Enabling debugging output"
			DEBUG = True
		elif i in ("-d","--demo"):
			print "Turning on demo mode"
			DEMO = True
		elif i in ("-f","--fast"):
			print "Turning on fast demo mode"	# This disables the random sleeps
			DEMO = True
			FASTDEMO = True
		elif i in ("-a","--apps"):
			print "Turning on apps only mode"	# Cluster and storage is setup, only run apps
			APPS_ONLY = True
		elif i in ("-h","--help"):
			usage()
			sys.exit()
		else:
			assert False, "unhandled option"

	# These are the pages defined in the glade file.  Increment here to match
	# the packing number of the page when adding/deleting pages
	PAGE_INTRO = 0
	PAGE_INSTALL_TYPE = PAGE_INTRO + 1
	PAGE_SIMPLE_INTRO = PAGE_INSTALL_TYPE + 1
	PAGE_MANUAL_INTRO = PAGE_SIMPLE_INTRO + 1
	PAGE_CLUSTER_INFO = PAGE_MANUAL_INTRO + 1
	PAGE_CLUSTER_NODES = PAGE_CLUSTER_INFO + 1
	PAGE_CLUSTER_FENCING_TYPES = PAGE_CLUSTER_NODES + 1
	PAGE_CLUSTER_FENCING = PAGE_CLUSTER_FENCING_TYPES + 1
	PAGE_CLUSTER_DRAC_NODES = PAGE_CLUSTER_FENCING + 1
	PAGE_STORAGE_TYPE = PAGE_CLUSTER_DRAC_NODES + 1
	PAGE_STORAGE_ISCSI = PAGE_STORAGE_TYPE + 1
	PAGE_CLUSTER_SUMMARY = PAGE_STORAGE_ISCSI + 1
	PAGE_CLUSTER_STATUS = PAGE_CLUSTER_SUMMARY + 1
	PAGE_STORAGE_CONFIG = PAGE_CLUSTER_STATUS + 1
	PAGE_DISCOVERY_STATUS = PAGE_STORAGE_CONFIG + 1
	PAGE_CONFIRM_STORAGE = PAGE_DISCOVERY_STATUS + 1
	PAGE_APPS = PAGE_CONFIRM_STORAGE + 1
	PAGE_APP_NFS = PAGE_APPS + 1
	PAGE_APP_WEB = PAGE_APP_NFS + 1
	PAGE_APP_SAMBA = PAGE_APP_WEB + 1
	PAGE_APP_VIRT = PAGE_APP_SAMBA + 1
	PAGE_APP_FTP = PAGE_APP_VIRT + 1
	PAGE_APPS_SUMMARY = PAGE_APP_FTP + 1
	PAGE_STORAGE_STATUS = PAGE_APPS_SUMMARY + 1
	PAGE_COMPLETED = PAGE_STORAGE_STATUS + 1

	def __init__(self):
		gtk.Assistant.__init__(self)
		if self.DEBUG: print "Starting dell-config-cluster-gui", _major_version + "." + _minor_version
		#Set the Glade file
		self.gladefile = "dell-config-cluster-gui.glade"
		self.wTree = gtk.glade.XML(self.gladefile) 
		self.dccg = self.wTree.get_widget('dell_config_cluster')
		#dccg.threads_init()
		self.dccg.set_size_request(750, 520)
		self.dccg.set_forward_page_func(self.get_next_page)
		self.dccg.set_title("Dell Config Cluster " + _major_version)

		#Create our dictionay and connect it
		dic = { "on_dell_config_cluster_cancel" : gtk.main_quit,
			"on_dell_config_cluster_prepare" : self.prepare_page,
			"on_dell_config_cluster_destory" : gtk.main_quit,
			"on_dell_config_cluster_close" : gtk.main_quit,
			"on_label_help_url_button_release_event" : self.launch_url,
			"on_md3000i_iscsi_url_button_release_event" : self.launch_url,
			"on_eqlogic_iscsi_url_button_release_event" : self.launch_url,
			"on_eqlogic_storage_config_url_button_release_event" : self.launch_url,
			"on_md3000i_storage_config_url_button_release_event" : self.launch_url,
			"on_md3000_storage_config_url_button_release_event" : self.launch_url,
			"on_checkbutton_drac_toggled" : self.checkbutton_drac_toggled,
			"on_checkbutton_drac_toggled" : self.checkbutton_drac_toggled,
			"on_checkbutton_storage_complete_toggled" : self.checkbutton_storage_complete_toggled,
			"on_cluster_name_changed" : self.verify_cluster_name_input,
			"on_cluster_name_focus_out_event" : self.verify_cluster_name_length,
			"on_cluster_password_changed" : self.cluster_password_input,
			"on_cluster_password_verify_changed" : self.cluster_password_verify_input,
			"on_button_detail_node_cluster1_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster2_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster3_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster4_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster5_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster6_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster7_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster8_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster9_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster10_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster11_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster12_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster13_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster14_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster15_clicked" : self.detail_button_pressed,
			"on_button_detail_node_cluster16_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery1_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery2_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery3_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery4_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery5_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery6_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery7_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery8_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery9_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery10_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery11_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery12_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery13_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery14_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery15_clicked" : self.detail_button_pressed,
			"on_button_detail_node_discovery16_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage1_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage2_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage3_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage4_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage5_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage6_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage7_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage8_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage9_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage10_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage11_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage12_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage13_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage14_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage15_clicked" : self.detail_button_pressed,
			"on_button_detail_node_storage16_clicked" : self.detail_button_pressed,
			"on_checkbutton_mgmt_toggled" : self.role_chooser,
			"on_checkbutton_deploy_toggled" : self.role_chooser,
			"on_checkbutton_cluster_toggled" : self.role_chooser,
			"on_checkbutton_reconfigure_toggled" : self.reconfigure_toggled,
			"on_checkbutton_ssh_keys_toggled" : self.use_ssh_keys,
			"on_checkbutton_node_passwords_same_toggled" : self.passwords_same,
			"on_checkbutton_drac_passwords_same_toggled" : self.passwords_same,
			"on_entry_ip_node1_changed" : self.validate_ip_input,
			"on_entry_ip_node2_changed" : self.validate_ip_input,
			"on_entry_ip_node3_changed" : self.validate_ip_input,
			"on_entry_ip_node4_changed" : self.validate_ip_input,
			"on_entry_ip_node5_changed" : self.validate_ip_input,
			"on_entry_ip_node6_changed" : self.validate_ip_input,
			"on_entry_ip_node7_changed" : self.validate_ip_input,
			"on_entry_ip_node8_changed" : self.validate_ip_input,
			"on_entry_ip_node9_changed" : self.validate_ip_input,
			"on_entry_ip_node10_changed" : self.validate_ip_input,
			"on_entry_ip_node11_changed" : self.validate_ip_input,
			"on_entry_ip_node12_changed" : self.validate_ip_input,
			"on_entry_ip_node13_changed" : self.validate_ip_input,
			"on_entry_ip_node14_changed" : self.validate_ip_input,
			"on_entry_ip_node15_changed" : self.validate_ip_input,
			"on_entry_ip_node16_changed" : self.validate_ip_input,
			"on_entry_ip_drac1_changed" : self.validate_ip_input,
			"on_entry_ip_drac2_changed" : self.validate_ip_input,
			"on_entry_ip_drac3_changed" : self.validate_ip_input,
			"on_entry_ip_drac4_changed" : self.validate_ip_input,
			"on_entry_ip_drac5_changed" : self.validate_ip_input,
			"on_entry_ip_drac6_changed" : self.validate_ip_input,
			"on_entry_ip_drac7_changed" : self.validate_ip_input,
			"on_entry_ip_drac8_changed" : self.validate_ip_input,
			"on_entry_ip_drac9_changed" : self.validate_ip_input,
			"on_entry_ip_drac10_changed" : self.validate_ip_input,
			"on_entry_ip_drac11_changed" : self.validate_ip_input,
			"on_entry_ip_drac12_changed" : self.validate_ip_input,
			"on_entry_ip_drac13_changed" : self.validate_ip_input,
			"on_entry_ip_drac14_changed" : self.validate_ip_input,
			"on_entry_ip_drac15_changed" : self.validate_ip_input,
			"on_entry_ip_drac16_changed" : self.validate_ip_input,
			"on_entry_ip_node1_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node2_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node3_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node4_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node5_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node6_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node7_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node8_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node9_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node10_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node11_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node12_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node13_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node14_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node15_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_node16_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac1_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac2_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac3_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac4_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac5_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac6_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac7_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac8_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac9_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac10_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac11_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac12_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac13_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac14_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac15_focus_out_event" : self.validate_ip_field,
			"on_entry_ip_drac16_focus_out_event" : self.validate_ip_field,
			"on_entry_storage_mount_point_changed" : self.validate_mount_point_input,
			"on_entry_nfs_ip_changed" : self.validate_nfs_input,
			"on_entry_nfs_dev_changed" : self.validate_nfs_input,
			"on_entry_nfs_mount_changed" : self.validate_nfs_input,
			"on_entry_nfs_clients_changed" : self.validate_nfs_input,
			"on_entry_nfs_client_opts_changed" : self.validate_nfs_input,
			"on_entry_nfs_ip_focus_out_event" : self.validate_nfs_field_handler,
			"on_entry_nfs_clients_focus_out_event" : self.validate_nfs_field_handler,
			"on_entry_nfs_client_opts_focus_out_event" : self.validate_nfs_field_handler,
			"on_entry_web_ip_focus_out_event" : self.validate_web_field_handler,
			"on_entry_smb_ip_focus_out_event" : self.validate_smb_field_handler,
			"on_entry_smb_workgroup_focus_out_event" : self.validate_smb_field_handler,
			"on_entry_virt_dev_focus_out_event" : self.validate_smb_field_handler,
			"on_entry_ftp_ip_focus_out_event" : self.validate_ftp_field_handler,
			"on_entry_web_ip_changed" : self.validate_web_input,
			"on_entry_web_dev_changed" : self.validate_web_input,
			"on_entry_web_mount_changed" : self.validate_web_input,
			"on_entry_smb_ip_changed" : self.validate_smb_input,
			"on_entry_smb_dev_changed" : self.validate_smb_input,
			"on_entry_smb_mount_changed" : self.validate_smb_input,
			"on_entry_smb_workgroup_changed" : self.validate_smb_input,
			"on_entry_virt_dev_changed" : self.validate_virt_input,
			"on_entry_virt_mount_changed" : self.validate_virt_input,
			"on_entry_ftp_ip_changed" : self.validate_ftp_input,
			"on_entry_ftp_dev_changed" : self.validate_ftp_input,
			"on_entry_ftp_mount_changed" : self.validate_ftp_input,
			"on_entry_iscsi_target_changed" : self.validate_iscsi_input,
			"on_label_storage_url_button_release_event" : self.launch_url,
			"on_label_redhat_docs_url_button_release_event" : self.launch_url,
			"on_label_additional_storage_config_url_button_release_event" : self.launch_url,
			"on_label_completed_cluster_docs_button_release_event" : self.launch_url,
			"on_cluster_minimal_url_button_release_event" : self.launch_url,
			"on_cluster_full_url_button_release_event" : self.launch_url,
			"on_label_survey_button_release_event" : self.launch_url,
			"on_checkbutton_use_drac_fencing_toggled" : self.change_drac_ipmi,
			"on_checkbutton_use_ipmi_fencing_toggled" : self.change_drac_ipmi }
		self.wTree.signal_autoconnect(dic)

		# Constants and other initialization actions
		# Fields only support up to 16 nodes, to add more, glade needs to be modified
		self.MAX_NODES = 16

		# setting up status bars
		self.cluster_info_context_id = self.wTree.get_widget('statusbar_cluster_info').get_context_id("Cluster Info")
		self.nfs_context_id = self.wTree.get_widget('statusbar_nfs').get_context_id("NFS")
		self.web_context_id = self.wTree.get_widget('statusbar_web').get_context_id("WEB")
		self.smb_context_id = self.wTree.get_widget('statusbar_smb').get_context_id("SAMBA")
		self.virt_context_id = self.wTree.get_widget('statusbar_virt').get_context_id("VIRT")
		self.ftp_context_id = self.wTree.get_widget('statusbar_ftp').get_context_id("FTP")

		# For some reason the glade file defaults are not being accepted for some of these, so setting them here
		self.wTree.get_widget('cluster_nodes').set_active(0)		# Sets default to 2 nodes
		self.wTree.get_widget('radiobutton_full_config').set_active(True)	# Sets default to full install
		
		# Set fake ips to use for demo mode.  Defaults to 8 node cluster, but fills in values for all 16
		if self.DEMO:
			demo_num = 8
			self.wTree.get_widget('cluster_password').set_text("password")
			self.wTree.get_widget('cluster_password_verify').set_text("password")
			self.wTree.get_widget('cluster_nodes').set_active(demo_num - 2)
			self.wTree.get_widget('entry_iscsi_target').set_text("example.iscsi.target")
			for node in range(1, self.MAX_NODES + 1):
				self.wTree.get_widget('entry_ip_node' + str(node)).set_text("192.168.100." + str(node))
				self.wTree.get_widget('entry_ip_drac' + str(node)).set_text("192.168.100." + str(node + 200))
			self.wTree.get_widget('entry_drac_gateway').set_text("192.168.100.254")
			self.wTree.get_widget('entry_drac_netmask').set_text("255.255.255.0")
			self.wTree.get_widget('checkbutton_nfs').set_active(True)
			self.wTree.get_widget('checkbutton_web').set_active(True)
			self.wTree.get_widget('checkbutton_samba').set_active(True)
			self.wTree.get_widget('checkbutton_ftp').set_active(True)
			self.wTree.get_widget('entry_nfs_ip').set_text("192.168.100.90")
			self.wTree.get_widget('entry_nfs_clients').set_text("192.168.0/24")
			self.wTree.get_widget('entry_nfs_client_opts').set_text("rw,no_root_squash,sync")
			self.wTree.get_widget('entry_web_ip').set_text("192.168.100.91")
			self.wTree.get_widget('entry_smb_ip').set_text("192.168.100.92")
			self.wTree.get_widget('entry_smb_workgroup').set_text("DELLCLUSTER")
			self.wTree.get_widget('entry_ftp_ip').set_text("192.168.100.93")
#			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_INFO), True)
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_NODES), True)
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_DRAC_NODES), True)

		# init the back end
		self.dcc = dellConfigCluster.DellConfigCluster()
		self.dcc.set_debug(self.DEBUG)
		self.dcc.set_test(self.DEMO)
		self.dcc.set_fasttest(self.FASTDEMO)
		self.dcc.set_apps_only(self.APPS_ONLY)

		# These pages should have forward button enabled by default, others are dynamically set
		self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_INTRO), True)
		self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_INSTALL_TYPE), True)
		self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_SIMPLE_INTRO), True)
		self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_MANUAL_INTRO), True)
		self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_FENCING_TYPES), True)
		self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_FENCING), True)
		self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_STORAGE_TYPE), True)
		self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_SUMMARY), True)
		self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APPS), True)
		self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APPS_SUMMARY), True)

	def prepare_page(self, assistant, page):	# This function is called before each page is shown
		if self.DEBUG: print "Running preparing functions for page: ", page
		if page == assistant.get_nth_page(self.PAGE_CLUSTER_NODES):
			self.init_node_table()
			self.wTree.get_widget('label_hostnames_node').hide()		# Hiding hostname entries since they are not doing anything
			for entry in range(1, self.MAX_NODES + 1):
				self.wTree.get_widget('entry_hostname_node' + str(entry)).hide()
			self.wTree.get_widget('scrolledwindow_nodes').queue_resize()	# This method will redraw the page, which is needed when widgets are hidden/shown
		elif page == assistant.get_nth_page(self.PAGE_CLUSTER_FENCING_TYPES):
			assistant.set_page_complete(page, True)
			if self.wTree.get_widget('checkbutton_drac').get_active():	# If the systems have DRACs, then hide the ipmi widgets
				self.wTree.get_widget('alignment_label_ipmi_fencing').hide()
				self.wTree.get_widget('alignment_checkbutton_ipmi_fencing').hide()
				self.wTree.get_widget('label_drac_fencing').show()
				self.wTree.get_widget('checkbutton_use_drac_fencing').show()
			else:								# if no DRACS, hide DRAC widgets
				self.wTree.get_widget('alignment_label_ipmi_fencing').show()
				self.wTree.get_widget('alignment_checkbutton_ipmi_fencing').show()
				self.wTree.get_widget('label_drac_fencing').hide()
				self.wTree.get_widget('checkbutton_use_drac_fencing').hide()
			self.wTree.get_widget('scrolledwindow_fencing_types').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_CLUSTER_DRAC_NODES):
			self.init_drac_table()
			self.wTree.get_widget('scrolledwindow_dracs').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_STORAGE_TYPE):
			if self.DEBUG: print "checking if more than 2 nodes are selected: ", self.wTree.get_widget('cluster_nodes').get_active() + 2
			if self.wTree.get_widget('cluster_nodes').get_active() > 0:
				if self.DEBUG: print "More than 2 nodes selected, disabling MD3000"
				self.wTree.get_widget('radiobutton_md3000').set_sensitive(False)
			else:
				if self.DEBUG: print "2 nodes selected, enabling MD3000"
				self.wTree.get_widget('radiobutton_md3000').set_sensitive(True)
		elif page == assistant.get_nth_page(self.PAGE_STORAGE_ISCSI):
			if self.wTree.get_widget('radiobutton_equallogic').get_active():
				if self.DEBUG: print "Swapping in eqlogic iscsi text based on choices"
				self.wTree.get_widget('vbox_iscsi_eqlogic').show()
				self.wTree.get_widget('vbox_iscsi_md3000i').hide()
			elif self.wTree.get_widget('radiobutton_md3000i').get_active():
				if self.DEBUG: print "Swapping in md3000i iscsi text based on choices"
				self.wTree.get_widget('vbox_iscsi_eqlogic').hide()
				self.wTree.get_widget('vbox_iscsi_md3000i').show()
			self.wTree.get_widget('scrolledwindow_iscsi').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_STORAGE_CONFIG):
			if self.wTree.get_widget('radiobutton_equallogic').get_active():
				if self.DEBUG: print "Swapping in eqlogic storage access control text based on choices"
				self.wTree.get_widget('vbox_eqlogic_storage_config').show()
				self.wTree.get_widget('vbox_md3000i_storage_config').hide()
				self.wTree.get_widget('vbox_md3000_storage_config').hide()
			elif self.wTree.get_widget('radiobutton_md3000i').get_active():
				if self.DEBUG: print "Swapping in md3000i storage access control text based on choices"
				self.wTree.get_widget('vbox_eqlogic_storage_config').hide()
				self.wTree.get_widget('vbox_md3000i_storage_config').show()
				self.wTree.get_widget('vbox_md3000_storage_config').hide()
			elif self.wTree.get_widget('radiobutton_md3000').get_active():
				if self.DEBUG: print "Swapping in md3000 storage access control text based on choices"
				self.wTree.get_widget('vbox_eqlogic_storage_config').hide()
				self.wTree.get_widget('vbox_md3000i_storage_config').hide()
				self.wTree.get_widget('vbox_md3000_storage_config').show()
			self.wTree.get_widget('scrolledwindow_storage_configuration').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_CLUSTER_SUMMARY):
			self.show_cluster_info()
			self.wTree.get_widget('scrolledwindow_summary').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_CLUSTER_STATUS):
			if self.wTree.get_widget('radiobutton_minimal_config').get_active():
				self.dccg.set_page_title(self.dccg.get_nth_page(self.PAGE_CLUSTER_STATUS), "1/1: Cluster Creation")
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_STATUS), False)
			self.dccg.set_page_type(self.dccg.get_nth_page(self.PAGE_CLUSTER_STATUS), gtk.ASSISTANT_PAGE_PROGRESS)
			if self.DEBUG: print "calling configure nodes"
			self.init_cluster_status_page()
			self.configure_cluster_nodes()
			self.wTree.get_widget('vbox_cluster_status').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_DISCOVERY_STATUS):
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_DISCOVERY_STATUS), False)
			self.dccg.set_page_type(self.dccg.get_nth_page(self.PAGE_DISCOVERY_STATUS), gtk.ASSISTANT_PAGE_PROGRESS)
			self.init_discovery_status_page()
			self.wTree.get_widget('vbox_discovery_status').queue_resize()
			self.dcc.nodes_verify_storage(self.cb_update_discovery_status, self.cb_update_discovery_percent, self.cb_update_discovery_finished)
		elif page == assistant.get_nth_page(self.PAGE_CONFIRM_STORAGE):
			self.wTree.get_widget('entry_storage_mount_point').set_text(self.wTree.get_widget('entry_storage_mount_point').get_text().rstrip(' /'))
			self.validate_mount_point_input(self.wTree.get_widget('entry_storage_mount_point'))
			self.wTree.get_widget('scrolledwindow_storage_status').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_STORAGE_STATUS):
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_STORAGE_STATUS), False)
			self.dccg.set_page_type(self.dccg.get_nth_page(self.PAGE_STORAGE_STATUS), gtk.ASSISTANT_PAGE_PROGRESS)
			if self.DEBUG: print "calling continue configure nodes, phase 2"
			self.init_storage_status_page()
			self.configure_storage_apps()
			self.wTree.get_widget('vbox_storage_status').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_APP_NFS):
			if not self.wTree.get_widget('entry_nfs_client_opts').get_text():
				self.wTree.get_widget('entry_nfs_client_opts').set_text("ro")
			self.wTree.get_widget('entry_nfs_dev').set_text(self.wTree.get_widget('entry_storage_dev').get_text())
			self.wTree.get_widget('entry_nfs_mount').set_text(self.wTree.get_widget('entry_storage_mount_point').get_text().rstrip(' /') + '/nfs')
			self.wTree.get_widget('vbox_nfs').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_APP_WEB):
			self.wTree.get_widget('entry_web_dev').set_text(self.wTree.get_widget('entry_storage_dev').get_text())
			self.wTree.get_widget('entry_web_mount').set_text(self.wTree.get_widget('entry_storage_mount_point').get_text().rstrip(' /') + '/web')
			self.wTree.get_widget('vbox_web').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_APP_SAMBA):
			self.wTree.get_widget('entry_smb_dev').set_text(self.wTree.get_widget('entry_storage_dev').get_text())
			self.wTree.get_widget('entry_smb_mount').set_text(self.wTree.get_widget('entry_storage_mount_point').get_text().rstrip(' /') + '/samba')
			self.wTree.get_widget('vbox_samba').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_APP_VIRT):
			self.wTree.get_widget('entry_virt_dev').set_text(self.wTree.get_widget('entry_storage_dev').get_text())
			self.wTree.get_widget('entry_virt_mount').set_text(self.wTree.get_widget('entry_storage_mount_point').get_text().rstrip(' /') + '/virt')
			self.wTree.get_widget('vbox_virt').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_APP_FTP):
			self.wTree.get_widget('entry_ftp_dev').set_text(self.wTree.get_widget('entry_storage_dev').get_text())
			self.wTree.get_widget('entry_ftp_mount').set_text(self.wTree.get_widget('entry_storage_mount_point').get_text().rstrip(' /') + '/ftp')
			self.wTree.get_widget('vbox_ftp').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_APPS_SUMMARY):
			self.show_apps_info()
			self.wTree.get_widget('scrolledwindow_apps_summary').queue_resize()
		elif page == assistant.get_nth_page(self.PAGE_COMPLETED):
			if self.wTree.get_widget('radiobutton_full_config').get_active():
				self.wTree.get_widget('vbox_completed_full').show()
				self.wTree.get_widget('vbox_completed_minimal').hide()
			elif self.wTree.get_widget('radiobutton_minimal_config').get_active():
				self.wTree.get_widget('vbox_completed_full').hide()
				self.wTree.get_widget('vbox_completed_minimal').show()
			self.wTree.get_widget('scrolledwindow_completed').queue_resize()

	def get_next_page(self, page_num_sent):			# Forward button page logic
		page_num = self.dccg.get_current_page()		# For some reason the page sent and current page are not always the same
		if page_num == self.PAGE_INSTALL_TYPE:
			if self.wTree.get_widget('radiobutton_full_config').get_active():
				if self.DEBUG: print "going to full_config page"
				return self.PAGE_SIMPLE_INTRO
			elif self.wTree.get_widget('radiobutton_minimal_config').get_active():
				if self.DEBUG: print "going to minimal_config page"
				return self.PAGE_MANUAL_INTRO
		elif page_num == self.PAGE_SIMPLE_INTRO:	# This is to skip the minimal_config page info
				return self.PAGE_CLUSTER_INFO
		elif page_num == self.PAGE_MANUAL_INTRO:	# For consitency, should not matter
				return self.PAGE_CLUSTER_INFO
		elif page_num == self.PAGE_CLUSTER_NODES:
			if self.wTree.get_widget('radiobutton_full_config').get_active():
				return self.PAGE_CLUSTER_FENCING_TYPES
			elif self.wTree.get_widget('radiobutton_minimal_config').get_active():
				if self.wTree.get_widget('checkbutton_drac').get_active():
					return self.PAGE_CLUSTER_DRAC_NODES
				else:
					return self.PAGE_STORAGE_TYPE
			else:
				return -1
		elif page_num == self.PAGE_CLUSTER_FENCING_TYPES:
			if self.wTree.get_widget('checkbutton_use_power_fencing').get_active():
				return self.PAGE_CLUSTER_FENCING
			elif self.wTree.get_widget('checkbutton_drac').get_active():
					return self.PAGE_CLUSTER_DRAC_NODES
			elif self.wTree.get_widget('checkbutton_use_ipmi_fencing').get_active():
					return self.PAGE_CLUSTER_DRAC_NODES
			else:
				return self.PAGE_STORAGE_TYPE
		elif page_num == self.PAGE_CLUSTER_FENCING:
			if self.wTree.get_widget('checkbutton_drac').get_active() or self.wTree.get_widget('checkbutton_use_ipmi_fencing').get_active():
				return self.PAGE_CLUSTER_DRAC_NODES
			else:
				return self.PAGE_STORAGE_TYPE
		elif page_num == self.PAGE_STORAGE_TYPE:
			if self.wTree.get_widget('radiobutton_md3000i').get_active() or self.wTree.get_widget('radiobutton_equallogic').get_active():
				return self.PAGE_STORAGE_ISCSI
			else:
				return self.PAGE_CLUSTER_SUMMARY
		elif page_num == self.PAGE_CLUSTER_STATUS:
			if self.wTree.get_widget('radiobutton_minimal_config').get_active():
				if self.DEBUG: print "minimal_config was selcted, jumping to end"
				return self.PAGE_COMPLETED
			else:
				if self.DEBUG: print "no minimal_config selected, next page"
				return self.PAGE_STORAGE_CONFIG
		elif page_num == self.PAGE_APPS:
			if self.wTree.get_widget('checkbutton_nfs').get_active():
				if self.DEBUG: print "DEBUG: nfs selected"
				return self.PAGE_APP_NFS
			elif self.wTree.get_widget('checkbutton_web').get_active():
				if self.DEBUG: print "DEBUG: web selected"
				return self.PAGE_APP_WEB
			elif self.wTree.get_widget('checkbutton_samba').get_active():
				if self.DEBUG: print "DEBUG: samba selected"
				return self.PAGE_APP_SAMBA
			elif self.wTree.get_widget('checkbutton_virt').get_active():
				if self.DEBUG: print "DEBUG: virt selected"
				return self.PAGE_APP_VIRT
			elif self.wTree.get_widget('checkbutton_ftp').get_active():
				if self.DEBUG: print "DEBUG: ftp selected"
				return self.PAGE_APP_FTP
			else:
				return self.PAGE_APPS_SUMMARY
		elif page_num == self.PAGE_APP_NFS:
			if self.wTree.get_widget('checkbutton_web').get_active():
				if self.DEBUG: print "DEBUG: web selected"
				return self.PAGE_APP_WEB
			elif self.wTree.get_widget('checkbutton_samba').get_active():
				if self.DEBUG: print "DEBUG: samba selected"
				return self.PAGE_APP_SAMBA
			elif self.wTree.get_widget('checkbutton_virt').get_active():
				if self.DEBUG: print "DEBUG: virt selected"
				return self.PAGE_APP_VIRT
			elif self.wTree.get_widget('checkbutton_ftp').get_active():
				if self.DEBUG: print "DEBUG: ftp selected"
				return self.PAGE_APP_FTP
			else:
				return self.PAGE_APPS_SUMMARY
		elif page_num == self.PAGE_APP_WEB:
			if self.wTree.get_widget('checkbutton_samba').get_active():
				if self.DEBUG: print "DEBUG: samba selected"
				return self.PAGE_APP_SAMBA
			elif self.wTree.get_widget('checkbutton_virt').get_active():
				if self.DEBUG: print "DEBUG: virt selected"
				return self.PAGE_APP_VIRT
			elif self.wTree.get_widget('checkbutton_ftp').get_active():
				if self.DEBUG: print "DEBUG: ftp selected"
				return self.PAGE_APP_FTP
			else:
				return self.PAGE_APPS_SUMMARY
		elif page_num == self.PAGE_APP_SAMBA:
			if self.wTree.get_widget('checkbutton_virt').get_active():
				if self.DEBUG: print "DEBUG: virt selected"
				return self.PAGE_APP_VIRT
			elif self.wTree.get_widget('checkbutton_ftp').get_active():
				if self.DEBUG: print "DEBUG: ftp selected"
				return self.PAGE_APP_FTP
			else:
				return self.PAGE_APPS_SUMMARY
		elif page_num == self.PAGE_APP_VIRT:
			if self.wTree.get_widget('checkbutton_ftp').get_active():
				if self.DEBUG: print "DEBUG: ftp selected"
				return self.PAGE_APP_FTP
			else:
				return self.PAGE_APPS_SUMMARY
		else:
			if self.DEBUG: print "DEBUG: No page match found, returning next page."
			return page_num + 1

	def configure_cluster_nodes(self):
		if self.DEBUG: print "calling clear nodes"
		self.dcc.clear_nodes()
		num_nodes = int(self.wTree.get_widget('cluster_nodes').get_active_text())
		
		if self.DEBUG: 
			print "cluster name: ", self.wTree.get_widget('cluster_name').get_text()
			print "cluster size: ", int(self.wTree.get_widget('cluster_nodes').get_active_text())
		if self.wTree.get_widget('radiobutton_full_config').get_active():
			if self.DEBUG: print "  Install type: full_config"
		elif self.wTree.get_widget('radiobutton_minimal_config').get_active():
			if self.DEBUG: print "  Install type: minimal_config"
			self.dcc.clear_apps()
			self.dcc.clear_fence()
		for i in range(1, num_nodes + 1):
#			node_name = self.wTree.get_widget('entry_hostname_node' + str(i)).get_text()	# Removed until hostnames are set in /etc/hosts
			if self.wTree.get_widget('checkbutton_node_passwords_same').get_active():
				# If they set this button, then all passwords are the first entry box
				password=self.wTree.get_widget('entry_password_node' + str(1)).get_text()
			else:
				password=self.wTree.get_widget('entry_password_node' + str(i)).get_text()
			if self.wTree.get_widget('checkbutton_drac_passwords_same').get_active():
				# Same here, all drac/ipmi passwords would equal first entry box
				drac_password=self.wTree.get_widget('entry_password_drac' + str(1)).get_text()
			else:
				drac_password=self.wTree.get_widget('entry_password_drac' + str(i)).get_text()
			self.dcc.add_node( name=self.wTree.get_widget('entry_ip_node' + str(i)).get_text(),	# Sending ip for now until hostsnames set
#					name=node_name,			# Removed until hostsnames are set in /etc/hosts
					password=password,
					private_ipaddr=self.wTree.get_widget('entry_ip_node' + str(i)).get_text())
			if self.wTree.get_widget('checkbutton_drac').get_active():
				self.dcc.add_node_drac(node_id=i,
						       ipaddr=self.wTree.get_widget('entry_ip_drac' + str(i)).get_text(),
						       user="root",
						       password=drac_password,
						       gateway=self.wTree.get_widget('entry_drac_gateway').get_text(),
						       netmask=self.wTree.get_widget('entry_drac_netmask').get_text() )
				if self.wTree.get_widget('checkbutton_use_power_fencing').get_active():
					self.dcc.set_primary_fence("nps")
					if self.wTree.get_widget('checkbutton_use_drac_fencing').get_active():
						self.dcc.set_secondary_fence("drac")
				elif self.wTree.get_widget('checkbutton_use_drac_fencing').get_active():
					self.dcc.set_primary_fence("drac")
			else:
				if self.wTree.get_widget('checkbutton_use_power_fencing').get_active():
					self.dcc.set_primary_fence("nps")
					if self.wTree.get_widget('checkbutton_use_ipmi_fencing').get_active():
						self.dcc.add_node_ipmi(node_id=i,
								       ipaddr=self.wTree.get_widget('entry_ip_drac' + str(i)).get_text(),
								       user="cluster",
								       password=drac_password,
								       gateway=self.wTree.get_widget('entry_drac_gateway').get_text(),
								       netmask=self.wTree.get_widget('entry_drac_netmask').get_text() )

						self.dcc.set_secondary_fence("ipmi")
						print "setting ipmi 2nd"
				elif self.wTree.get_widget('checkbutton_use_ipmi_fencing').get_active():
					self.dcc.add_node_ipmi(node_id=i,
							ipaddr=self.wTree.get_widget('entry_ip_drac' + str(i)).get_text(),
							user="cluster",
							password=drac_password,
							gateway=self.wTree.get_widget('entry_drac_gateway').get_text(),
							netmask=self.wTree.get_widget('entry_drac_netmask').get_text())
					self.dcc.set_primary_fence("ipmi")
			if self.DEBUG: print " * DRAC ip for system is: ", self.wTree.get_widget('entry_ip_drac' + str(i)).get_text()
		self.dcc.set_cluster_name(self.wTree.get_widget('cluster_name').get_text())
		self.dcc.set_cluster_password(self.wTree.get_widget('cluster_password').get_text())

		if self.wTree.get_widget('radiobutton_md3000').get_active():
			self.dcc.set_storage( 'md3000' )
		elif self.wTree.get_widget('radiobutton_md3000i').get_active():
			self.dcc.set_storage( 'md3000i', iscsi_target=self.wTree.get_widget('entry_iscsi_target').get_text() )
		elif self.wTree.get_widget('radiobutton_equallogic').get_active():
			self.dcc.set_storage( 'el', iscsi_target=self.wTree.get_widget('entry_iscsi_target').get_text() )
		self.dcc.nodes_configure_all(self.cb_update_cluster_status, self.cb_update_cluster_percent, self.cb_update_cluster_finished)

	def configure_storage_apps(self):
		if self.DEBUG: print "calling clear apps"
		self.dcc.clear_apps()
		if self.wTree.get_widget('checkbutton_nfs').get_active():
			self.dcc.add_app( self.dcc.KEY_NFS,
					  ip=self.wTree.get_widget('entry_nfs_ip').get_text(),
					  dev=self.wTree.get_widget('entry_nfs_dev').get_text(),
#					  mount=self.wTree.get_widget('entry_nfs_mount').get_text(),
					  mount=self.wTree.get_widget('entry_storage_mount_point').get_text(),
					  clients=self.wTree.get_widget('entry_nfs_clients').get_text(),
					  client_opts=self.wTree.get_widget('entry_nfs_client_opts').get_text() )
		if self.wTree.get_widget('checkbutton_web').get_active():
			self.dcc.add_app( self.dcc.KEY_WEB,
					  ip=self.wTree.get_widget('entry_web_ip').get_text(),
					  dev=self.wTree.get_widget('entry_web_dev').get_text(), 
#					  mount=self.wTree.get_widget('entry_web_mount').get_text() )
					  mount=self.wTree.get_widget('entry_storage_mount_point').get_text() )
		if self.wTree.get_widget('checkbutton_samba').get_active():
			self.dcc.add_app( self.dcc.KEY_SAMBA,
					  ip=self.wTree.get_widget('entry_smb_ip').get_text(),
					  dev=self.wTree.get_widget('entry_smb_dev').get_text(),
#					  mount=self.wTree.get_widget('entry_smb_mount').get_text(),
					  mount=self.wTree.get_widget('entry_storage_mount_point').get_text(),
					  workgroup=self.wTree.get_widget('entry_smb_workgroup').get_text() )
			self.wTree.get_widget('summary_samba').set_text("configure")
		if self.wTree.get_widget('checkbutton_virt').get_active():
			self.dcc.add_app( self.dcc.KEY_VIRT,
					  dev=self.wTree.get_widget('entry_virt_dev').get_text(),
#					  mount=self.wTree.get_widget('entry_virt_mount').get_text() )
					  mount=self.wTree.get_widget('entry_storage_mount_point').get_text() )
		if self.wTree.get_widget('checkbutton_ftp').get_active():
			self.dcc.add_app( self.dcc.KEY_FTP,
					  ip=self.wTree.get_widget('entry_ftp_ip').get_text(),
					  dev=self.wTree.get_widget('entry_ftp_dev').get_text(),
#					  mount=self.wTree.get_widget('entry_ftp_mount').get_text() )
					  mount=self.wTree.get_widget('entry_storage_mount_point').get_text() )
		self.dcc.nodes_continue_config_all(self.cb_update_storage_status, self.cb_update_storage_percent, self.cb_update_storage_finished)
		
	def detail_button_pressed(self, button):
		# iterate through map built in init_cluster_status_page, set key to matching button
		node_button_map = {}
		node_details = {}
		if self.dccg.get_current_page() == self.PAGE_CLUSTER_STATUS:
			page = "cluster"
			node_detail_button_map = self.cluster_node_detail_button_map
			node_details = self.cluster_node_details
		elif self.dccg.get_current_page() == self.PAGE_STORAGE_STATUS:
			page = "storage"
			node_detail_button_map = self.storage_node_detail_button_map
			node_details = self.storage_node_details
		elif self.dccg.get_current_page() == self.PAGE_DISCOVERY_STATUS:
			page = "discovery"
			node_detail_button_map = self.discovery_node_detail_button_map
			node_details = self.discovery_node_details
		else:
			if self.DEBUG: print "Epic Fail!"
			return False
		for key in node_detail_button_map:
			if self.DEBUG:
				print "key is: ", key
				print "button name is: ", button.get_name()
				print "node_detail_button_map:", node_detail_button_map
			if button.get_name() == node_detail_button_map[key]:
				node = key
		if self.DEBUG:
			print "operating on : node:", node, "value of:", node_detail_button_map[node]
			print "button pressed, state is: ", node_details[node]
		if node_details[node]:		# Dict built in init_cluster_status_page to track toggles
			# Button "hide" was pressed, show progress bar and hide text view
			if self.DEBUG: print "setting less details for ", button.get_name()
			node_details[node] = False
			self.wTree.get_widget('scrolledwindow_textview_node_' + page + str(node)).hide()
			self.wTree.get_widget('alignment_progressbar_node_' + page + str(node)).show()
			self.wTree.get_widget('alignment_label_button_node_' + page + str(node)).set_padding(0,0,0,10)
			button.set_label('Details')
		else:
			# Button "details" was pressed, hide progress bar and show text view
			if self.DEBUG: print "setting more details for ", button.get_name()
			node_details[node] = True
			self.wTree.get_widget('scrolledwindow_textview_node_' + page + str(node)).show()
			self.wTree.get_widget('alignment_progressbar_node_' + page + str(node)).hide()
			self.wTree.get_widget('alignment_label_button_node_' + page + str(node)).set_padding(0,0,0,20)
			button.set_label(" Hide ")

	def cb_update_cluster_status(self, id, node, severity, message):
		return self._cb_update_status(id, node, severity, message, type="cluster")

	def cb_update_discovery_status(self, id, node, severity, message):
		return self._cb_update_status(id, node, severity, message, type="discovery")

	def cb_update_storage_status(self, id, node, severity, message):
		return self._cb_update_status(id, node, severity, message, type="storage")

	def _cb_update_status(self, id, node, severity, message, type=""):
		if severity == self.dcc.FAIL:
			severity_msg = "FAIL"
		elif severity == self.dcc.WARN:
			severity_msg = "WARN"
		elif severity == self.dcc.INFO:
			severity_msg = "INFO"
		else:
			severity_msg = str(severity)
		if not type: return False
		if self.DEBUG: print "cb_status: " + type + "status: " + node  + "(" + str(id) + ")" + " [" + severity_msg + "]: " + message
		gtk.gdk.threads_enter()
		if not id == 0:
			if severity == self.dcc.TICK:
				if self.DEBUG: print "cb_status: tick was sent, only pulsing bar"
				self.wTree.get_widget('progressbar_node_' + type + str(id)).pulse()
			elif severity in (self.dcc.FAIL, self.dcc.WARN, self.dcc.INFO) and message:
				if self.DEBUG: print "cb_status: Writing this to progressbar, details, and statusbar: ", repr(message)
				now = time.ctime()
				now_parsed = time.strptime(now)
				if self.DEBUG: print time.strftime("%a %b %d %H:%M:%S %Y", now_parsed)
				text_iter = self.text_buffers[id].get_end_iter()
				self.text_buffers[id].insert(text_iter, time.strftime("%H:%M:%S ", now_parsed) + " (" + node + ")" + ": "  + " [" + severity_msg + "]: " + message + "\n")
				self.wTree.get_widget('progressbar_node_' + type + str(id)).set_text(message)
				self.message_id = self.wTree.get_widget('statusbar_' + type + '_status').push(self.status_context_id, "[" + severity_msg + "] (" + node + "): " + message)
			else:
				if self.DEBUG: print "cb_status: message was empty, not writing anything"
		else:
			self.message_id = self.wTree.get_widget('statusbar_' + type + '_status').push(self.status_context_id, "All nodes" + " [" + severity_msg + "]: " + message)
		gtk.gdk.threads_leave()

	def cb_update_cluster_percent(self, id, percent):
		if self.DEBUG: print "cb_percent for cluster: received fraction of: ", percent/100.0
		return self._cb_update_percent(id, percent, type="cluster")

	def cb_update_discovery_percent(self, id, percent):
		if self.DEBUG: print "cb_percent for discovery: received fraction of: ", percent/100.0
		return self._cb_update_percent(id, percent, type="discovery")

	def cb_update_storage_percent(self, id, percent):
		if self.DEBUG: print "cb_percent for storage: received fraction of: ", percent/100.0
		return self._cb_update_percent(id, percent, type="storage")

	def _cb_update_percent(self, id, percent, type=""):
		if not type: return False
		if self.DEBUG: print "cb_percent: received fraction of: ", percent/100.0, " for", id, " of type: ", type
		self.wTree.get_widget('progressbar_node_' + type + str(id)).set_fraction(percent/100.0)

	def cb_update_cluster_finished(self, ok):
		return self._cb_update_finished(ok, type="cluster")

	def cb_update_discovery_finished(self, ok):
		return self._cb_update_finished(ok, type="discovery")

	def cb_update_storage_finished(self, ok):
		return self._cb_update_finished(ok, type="storage")

	def _cb_update_finished(self, ok, type=""):
		if not type: return False
		self.wTree.get_widget('vbox_' + type + '_status').queue_resize()
		if self.DEBUG: print "cb_update_finished callback: FINISHED: action returned: " + str(ok)
		gtk.gdk.threads_enter()
		if ok: 
			if self.DEBUG: print "SUCCESS: Setting page " + type + " status to complete"
			if type == "cluster":
				self.dccg.set_page_type(self.dccg.get_nth_page(self.PAGE_CLUSTER_STATUS), gtk.ASSISTANT_PAGE_INTRO)
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_STATUS), True)
				luci_url = '<span color="blue" underline="single">' + str(self.dcc.get_luci_url()) + '</span>'
				if self.wTree.get_widget('radiobutton_full_config').get_active():
					self.wTree.get_widget('cluster_full_url').set_markup(luci_url)
				elif self.wTree.get_widget('radiobutton_minimal_config').get_active():
					self.wTree.get_widget('cluster_minimal_url').set_markup(luci_url)
			elif type == "discovery":
				storage = self.dcc.get_cluster_storage_devs()[0]
				self.wTree.get_widget('entry_storage_scsi_id').set_text(storage["scsi_id"])
				self.wTree.get_widget('entry_storage_size').set_text(str(int(storage["size"]) / 2048) + " MiB")
				self.wTree.get_widget('entry_storage_dev').set_text(storage["logical_device"])
				self.dccg.set_page_type(self.dccg.get_nth_page(self.PAGE_DISCOVERY_STATUS), gtk.ASSISTANT_PAGE_INTRO)
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_DISCOVERY_STATUS), True)
			elif type == "storage":
				self.dccg.set_page_type(self.dccg.get_nth_page(self.PAGE_STORAGE_STATUS), gtk.ASSISTANT_PAGE_INTRO)
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_STORAGE_STATUS), True)
			self.message_id = self.wTree.get_widget('statusbar_' + type + '_status').push(self.status_context_id, "All nodes completed " + type + " configuration successfully.")
		else:
			if self.DEBUG: print "WARNING: Page " + type + " status complete, but action failed."
			if type == "cluster":
				self.dccg.set_page_type(self.dccg.get_nth_page(self.PAGE_CLUSTER_STATUS), gtk.ASSISTANT_PAGE_CONTENT)
			elif type == "discovery":
				self.dccg.set_page_type(self.dccg.get_nth_page(self.PAGE_DISCOVERY_STATUS), gtk.ASSISTANT_PAGE_CONTENT)
			elif type == "storage":
				self.dccg.set_page_type(self.dccg.get_nth_page(self.PAGE_STORAGE_STATUS), gtk.ASSISTANT_PAGE_CONTENT)
			self.message_id = self.wTree.get_widget('statusbar_' + type + '_status').push(self.status_context_id, "A failure occurred for " + type + " configuration, check details on failed nodes.")
		gtk.gdk.threads_leave()

	def init_cluster_status_page(self):
		num_nodes = int(self.wTree.get_widget('cluster_nodes').get_active_text())
		self.text_buffers = {}
		self.cluster_node_detail_button_map = {}
		self.cluster_node_details = {}
		for node in range(num_nodes + 1, self.MAX_NODES + 1):		# Hide boxes for not selected nodes
			if self.DEBUG: print "hiding widget for node: ", node
			self.wTree.get_widget('hbox_status_node_cluster' + str(node)).hide()
		for node in range(1, num_nodes + 1):		# Show status up to selected num_nodes
			self.wTree.get_widget('alignment_label_button_node_cluster' + str(node)).set_padding(0,0,0,10)
			self.text_buffers[node] = self.wTree.get_widget('textview_node_cluster' + str(node)).get_buffer()
			if self.DEBUG: print "showing scrolled window for node: ", node
			self.wTree.get_widget('hbox_status_node_cluster' + str(node)).show()
			self.cluster_node_detail_button_map[node] = "button_detail_node_cluster" + str(node)
			self.cluster_node_details[node] = False

		#setup status bar
		self.status_context_id = self.wTree.get_widget('statusbar_cluster_status').get_context_id("Cluster Status")

	def init_discovery_status_page(self):
		num_nodes = int(self.wTree.get_widget('cluster_nodes').get_active_text())
		self.text_buffers = {}
		self.discovery_node_detail_button_map = {}
		self.discovery_node_details = {}
		for node in range(num_nodes + 1, self.MAX_NODES + 1):		# Hide boxes for not selected num_nodes
			if self.DEBUG: print "hiding scrolled window for node: ", node
			self.wTree.get_widget('hbox_status_node_discovery' + str(node)).hide()
		for node in range(1, num_nodes + 1):		# Show status up to selected num_nodes
			self.wTree.get_widget('alignment_label_button_node_discovery' + str(node)).set_padding(0,0,0,10)
			self.text_buffers[node] = self.wTree.get_widget('textview_node_discovery' + str(node)).get_buffer()
			if self.DEBUG: print "showing scrolled window for node: ", node
			self.wTree.get_widget('hbox_status_node_discovery' + str(node)).show()
			self.discovery_node_detail_button_map[node] = "button_detail_node_discovery" + str(node)
			self.discovery_node_details[node] = False

		#setup status bar
		self.status_context_id = self.wTree.get_widget('statusbar_discovery_status').get_context_id("Storage Status")

	def init_storage_status_page(self):
		num_nodes = int(self.wTree.get_widget('cluster_nodes').get_active_text())
		self.text_buffers = {}
		self.storage_node_detail_button_map = {}
		self.storage_node_details = {}
		for node in range(num_nodes + 1, self.MAX_NODES + 1):		# Hide boxes for not selected nodes
			if self.DEBUG: print "hiding scrolled window for node: ", node
			self.wTree.get_widget('hbox_status_node_storage' + str(node)).hide()
		for node in range(1, num_nodes + 1):		# Show status up to selected num_nodes
			self.wTree.get_widget('alignment_label_button_node_storage' + str(node)).set_padding(0,0,0,10)
			self.text_buffers[node] = self.wTree.get_widget('textview_node_storage' + str(node)).get_buffer()
			if self.DEBUG: print "showing scrolled window for node: ", node
			self.wTree.get_widget('hbox_status_node_storage' + str(node)).show()
			self.storage_node_detail_button_map[node] = "button_detail_node_storage" + str(node)
			self.storage_node_details[node] = False

		#setup status bar
		self.status_context_id = self.wTree.get_widget('statusbar_storage_status').get_context_id("Storage Status")

	def init_node_table(self):
		num_nodes = int(self.wTree.get_widget('cluster_nodes').get_active_text())
		cluster_name = self.wTree.get_widget('cluster_name').get_text()
		self.nodes_context_id = self.wTree.get_widget('statusbar_nodes').get_context_id("Cluster Node Entry")
		for i in range(1, num_nodes + 1):		# Show node fields up to selected num_nodes
			if self.DEBUG: print "showing:", 'node' + str(i)
			self.wTree.get_widget('label_node' + str(i)).show()
			self.wTree.get_widget('entry_hostname_node' + str(i)).show()
			self.wTree.get_widget('entry_ip_node' + str(i)).show()
			self.wTree.get_widget('entry_password_node' + str(i)).show()
			if not self.wTree.get_widget('entry_hostname_node' + str(i)).get_text():
				node_name = "%s" % cluster_name + "%02i" % i
				self.wTree.get_widget('entry_hostname_node' + str(i)).set_width_chars(len(node_name))
				self.wTree.get_widget('entry_hostname_node' + str(i)).set_text(node_name)
		for i in range(num_nodes + 1, self.MAX_NODES + 1):	# hides the remaining nodes
			if self.DEBUG: print "hiding:", 'node' + str(i)
			self.wTree.get_widget('label_node' + str(i)).hide()
			self.wTree.get_widget('entry_hostname_node' + str(i)).hide()
			self.wTree.get_widget('entry_ip_node' + str(i)).hide()
			self.wTree.get_widget('entry_password_node' + str(i)).hide()
		self.passwords_same(self.wTree.get_widget('checkbutton_node_passwords_same'))

	def init_drac_table(self):
		num_nodes = int(self.wTree.get_widget('cluster_nodes').get_active_text())
		self.dracs_context_id = self.wTree.get_widget('statusbar_dracs').get_context_id("Cluster Drac Entry")
		for i in range(1, num_nodes + 1):		# Show node fields up to selected num_nodes
			if self.DEBUG: print "showing:", 'drac' + str(i)
			self.wTree.get_widget('label_drac' + str(i)).show()
			self.wTree.get_widget('entry_ip_drac' + str(i)).show()
			self.wTree.get_widget('entry_password_drac' + str(i)).show()
#			if self.wTree.get_widget('drac_network').get_text():
#				self.wTree.get_widget('entry_ip_drac' + str(i)).set_text(self.wTree.get_widget('drac_network').get_text() + "." + self.wTree.get_widget('entry_ip_node' + str(i)).get_text().split('.')[3])		# Hard to read, yes, but this grabs the corresponding ip, splits it by '.', and uses the last octet
		for i in range(num_nodes + 1, self.MAX_NODES + 1):	# hides the remaining nodes
			if self.DEBUG: print "hiding:", 'drac' + str(i)
			self.wTree.get_widget('label_drac' + str(i)).hide()
			self.wTree.get_widget('entry_ip_drac' + str(i)).hide()
			self.wTree.get_widget('entry_password_drac' + str(i)).hide()
		self.passwords_same(self.wTree.get_widget('checkbutton_drac_passwords_same'))

	def use_ssh_keys(self, state):
		num_nodes = int(self.wTree.get_widget('cluster_nodes').get_active_text())
		if state.get_active():
			for nodes in range(1, num_nodes + 1):
				self.wTree.get_widget('entry_password_node' + str(nodes)).set_sensitive(False)
			self.wTree.get_widget('checkbutton_node_passwords_same').set_sensitive(False)
			self.wTree.get_widget('entry_password_node1').set_text("")
		else:
			self.wTree.get_widget('checkbutton_node_passwords_same').set_sensitive(True)
			self.wTree.get_widget('entry_password_node1').set_sensitive(True) 		# In case this was disabled through toggle
			self.passwords_same(self.wTree.get_widget('checkbutton_node_passwords_same'))	# Now enable all other fields

	def checkbutton_drac_toggled(self, state):
		if state:
			self.wTree.get_widget('checkbutton_use_ipmi_fencing').set_active(False)
		else:
			self.wTree.get_widget('checkbutton_use_drac_fencing').set_active(False)

	def passwords_same(self, state):
		# Toggle sensitivity (i.e. grey out or interactive) based on checkbutton toggle
		num_nodes = int(self.wTree.get_widget('cluster_nodes').get_active_text())
		if state.get_name() == "checkbutton_drac_passwords_same":
			widget = "drac"
		elif state.get_name() == "checkbutton_node_passwords_same":
			widget = "node"
		if state.get_active():
			if self.DEBUG: print "passwords same"
			for node in range(2, num_nodes + 1):			# Leave first entry enabled, as it will be applied to all
				if self.DEBUG: print "setting non-editable:", widget + str(node) + '_password'
				self.wTree.get_widget('entry_password_' + widget + str(node)).set_sensitive(False)
		else:
			if self.DEBUG: print "passwords are different"
			for node in range(2, num_nodes + 1):			# Enable all now
				if self.DEBUG: print "setting editable:", widget + str(node) + '_password'
				self.wTree.get_widget('entry_password_' + widget + str(node)).set_sensitive(True)

	def change_drac_ipmi(self, state):
		if state.get_name() == "checkbutton_use_drac_fencing":
			if state.get_active():
				if self.DEBUG: print "swapping in drac info"
				self.wTree.get_widget('label_drac_ipmi').set_text("Enter cluster DRAC information:")
				self.dccg.set_page_title(self.dccg.get_nth_page(self.PAGE_CLUSTER_DRAC_NODES), "DRAC Configuration")
		elif state.get_name() == "checkbutton_use_ipmi_fencing":
			if state.get_active():
				if self.DEBUG: print "swapping in ipmi info"
				self.wTree.get_widget('label_drac_ipmi').set_text("Enter cluster ipmi information:")
				self.dccg.set_page_title(self.dccg.get_nth_page(self.PAGE_CLUSTER_DRAC_NODES), "IPMI Configuration")

	def auto_fill_ips(self, ip):
		num_nodes = int(self.wTree.get_widget('cluster_nodes').get_active_text())
		network = ip.get_text().split('.')
		network = network[0] + "." + network[1] + "." + network[2] + "."
		if ip.get_name() == "entry_ip_node1":
			widget = "node"
		elif ip.get_name() == "entry_ip_drac1":
			widget = "drac"
		for node in range(2, num_nodes + 1):
			if not self.wTree.get_widget('entry_ip_' + widget + str(node)).get_text():
				self.wTree.get_widget('entry_ip_' + widget + str(node)).set_text(network)
	
	def validate_ip_field(self, ip, event):	# this function will be called on the 'focus out' event.  The sole purpose of this func is to color invalid ip fields red, or white if they are valid
		current_page = self.dccg.get_current_page()
		if current_page == self.PAGE_CLUSTER_NODES:
			node_type = "node"
		elif current_page == self.PAGE_CLUSTER_DRAC_NODES:
			node_type = "drac"
		else:
			print "ERROR: Page could not be matched in validate_ip_field."
			return False
		if self.valid_ip(ip.get_text()):
			if self.DEBUG: print "Entry of '", ip.get_text(), "' is a valid IPv4 address."
			self.message_id = self.wTree.get_widget('statusbar_' + node_type + "s").push(self.nodes_context_id, "")
			if ip.get_name() == "entry_ip_node1" or ip.get_name() == "entry_ip_drac1": 
				self.auto_fill_ips(ip)
			self.validate_all_ips()
		else:
			ip.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
			print "WARNING: Entry of ", repr(ip.get_text()), " is not a valid IPv4 address."
			self.message_id = self.wTree.get_widget('statusbar_' + node_type + "s").push(self.nodes_context_id, "Invalid IP Address entry!")
			self.dccg.set_page_complete(self.dccg.get_nth_page(current_page), False)
		return False

	def verify_cluster_name_input(self, name):		# verify text as entered
		if self.valid_name(name.get_text()):
			if self.verify_cluster_name_length(name, None):
				self.validate_all_cluster_info_fields()
				return True
			else:
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_INFO), False)
				return False
		else:
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_INFO), False)
			return False

	def verify_cluster_name_length(self, name, event):	# verify text on focus-out event
		if self.valid_name(name.get_text()):
			if len(name.get_text()) > 15:
				# Cluster name must be 15 characters or less
				self.cluster_info_message_id = self.wTree.get_widget('statusbar_cluster_info').push(self.cluster_info_context_id, "Cluster name must be 15 characters or less!")
				name.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_INFO), False)
				return False
			else:
				self.cluster_info_message_id = self.wTree.get_widget('statusbar_cluster_info').push(self.cluster_info_context_id, "")
				name.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				if event:
					self.validate_all_cluster_info_fields()
				else:
					return True
		else:
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_INFO), False)
			return False

	def cluster_password_input(self, input):	# fuctions called as text is entered
		if input.get_text():
			if not len(input.get_text()) >= 6:
				# Passwords need to be 6 charcters or more
				input.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
				self.cluster_info_message_id = self.wTree.get_widget('statusbar_cluster_info').push(self.cluster_info_context_id, "Passwords must be at least 6 characters long!")
			else:
				input.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				self.cluster_info_message_id = self.wTree.get_widget('statusbar_cluster_info').push(self.cluster_info_context_id, "")
				self.verify_passwords(input, self.wTree.get_widget('cluster_password_verify'))
		else:
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_INFO), False)
			return False
			
	def cluster_password_verify_input(self, input):
		if self.wTree.get_widget('cluster_password_verify').get_text():
			self.verify_passwords(self.wTree.get_widget('cluster_password'), input)
		else:	
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_INFO), False)
			return False

	def verify_passwords(self, password, verify, called_from_validate_all=False):
		# The color flag will be used to highlight the box red 
		for field in (password, verify):
			if not field.get_text():
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_INFO), False)
				return False
		if password.get_text() == verify.get_text():
			verify.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
			self.cluster_info_message_id = self.wTree.get_widget('statusbar_cluster_info').push(self.cluster_info_context_id, "Passwords match.")
			if not called_from_validate_all:
				self.validate_all_cluster_info_fields()
			else:
				return True
		else:
			verify.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
			self.cluster_info_message_id = self.wTree.get_widget('statusbar_cluster_info').push(self.cluster_info_context_id, "Passwords do not match!")
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_INFO), False)
			return False

	def validate_all_cluster_info_fields(self):
		# check all fields before enabling forward button
		page = self.PAGE_CLUSTER_INFO
		name = self.wTree.get_widget('cluster_name')
		password = self.wTree.get_widget('cluster_password')
		verify = self.wTree.get_widget('cluster_password')

		if self.verify_cluster_name_length(name, None) and self.verify_passwords(password, verify, called_from_validate_all=True):
			self.dccg.set_page_complete(self.dccg.get_nth_page(page), True)
		else:
			self.dccg.set_page_complete(self.dccg.get_nth_page(page), False)

	def valid_mount_point(self, name_str):
		# Probably need better checks, but this works for now
		if self.DEBUG: print repr(name_str)
		if name_str[0] != "/":	# Make sure it starts with a '/'
			return False
		elif len(name_str) < 2:	# Makes sure it's not just '/'
			return False
		else:			# Assume it's good
			return True

	def valid_workgroup_len(self, name_str):
		if len(name_str) > 15:
			return False
		elif len(name_str) < 1:
			return False
		else:
			return True

	def valid_workgroup(self, name_str):
		name_pattern = "[^\w\-]"
		if re.search(name_pattern, name_str):
			return False
		else:
			return True

	def validate_nfs_client(self, client_str):
		pattern = "[^\w,\*,\?,\.,\/]"
		if re.search(pattern, client_str):
			if self.DEBUG: print "not a valid client: ", client_str
			return False
		else:
			return True

	def valid_name(self, name_str):
		name_pattern = "[^\w\-\.]"
		if re.search(name_pattern, name_str):
			if self.DEBUG: print "invalid match", name_str
			self.cluster_info_message_id = self.wTree.get_widget('statusbar_cluster_info').push(self.cluster_info_context_id, "Cluster name must use valid characters!")
			self.wTree.get_widget('cluster_name').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
			return False
		else:
			return True

	def validate_ip_input(self, input):	# fuctions called as ip text is entered
		# validate as text is changed, once a valid ip, then called validate_all_ips
		if self.valid_ip(input.get_text()):
			if self.DEBUG: print "Calling validate_all_ips"
			self.validate_all_ips()
		else:
			if self.DEBUG: print "IP input not valid yet"
			return False

	def validate_all_ips(self):	#this func will create a list of all ips, and check for dupes, and validate all fields
		current_page = self.dccg.get_current_page()
		if current_page == self.PAGE_CLUSTER_NODES:
			node_type = "node"
			if self.DEBUG: print "Validating all ips on Cluster Node entry page"
		elif current_page == self.PAGE_CLUSTER_DRAC_NODES:
			node_type = "drac"
			if self.DEBUG: print "Validating all ips on Drac/IPMI Node entry page"
		else:
			return False
		num_nodes = int(self.wTree.get_widget('cluster_nodes').get_active_text())
		all_ips = []
		for node in range(1, num_nodes + 1):		# Check ip addresses, build a list of all valid entries
			if self.DEBUG: print "checking node", node
			if self.valid_ip(self.wTree.get_widget('entry_ip_' + node_type + str(node)).get_text()):
				if self.DEBUG: "Node", node, " is a valid entry, adding to list"
				all_ips.append(self.wTree.get_widget('entry_ip_' + node_type + str(node)).get_text())
			else:
				if self.DEBUG: "Node", node, " is not yet valid, exiting validate check."
				all_ips = []
				return False
		if self.DEBUG: print "List of all ips:", all_ips
		check_ips = []
		for node in range(num_nodes):		# check for dupes
			if all_ips[node] in check_ips:
				print "ERROR: Duplicate ip found for node", str(node + 1)
				self.message_id = self.wTree.get_widget('statusbar_' + node_type + "s").push(self.nodes_context_id, "Duplicate IP Adderss detected for node" + str(node + 1) + "!")
				for dupe in range(num_nodes):
					if all_ips[node] == all_ips[dupe]:
						self.wTree.get_widget('entry_ip_' + node_type + str(dupe + 1)).modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("yellow"))
				self.wTree.get_widget('entry_ip_' + node_type + str(node + 1)).modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("yellow"))
				self.dccg.set_page_complete(self.dccg.get_nth_page(current_page), False)
				return False
			else:
				check_ips.append(all_ips[node])
				self.wTree.get_widget('entry_ip_' + node_type + str(node + 1)).modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
		if self.DEBUG: print "assuming that all ips are sane and not duplicated"
		self.message_id = self.wTree.get_widget('statusbar_' + node_type + "s").push(self.nodes_context_id, "All IP Address are valid entries.")
		self.dccg.set_page_complete(self.dccg.get_nth_page(current_page), True)
		return True

	def show_cluster_info(self):
		self.wTree.get_widget('summary_cluster_name').set_text(self.wTree.get_widget('cluster_name').get_text())
		self.wTree.get_widget('summary_cluster_nodes').set_text(self.wTree.get_widget('cluster_nodes').get_active_text())
		if self.wTree.get_widget('checkbutton_drac').get_active():
			self.wTree.get_widget('summary_drac_ipmi_config').set_text("yes")
		else:
			self.wTree.get_widget('summary_drac_ipmi_config').set_text("no")
		self.wTree.get_widget('summary_cluster_nodes').set_text(self.wTree.get_widget('cluster_nodes').get_active_text())
		if self.wTree.get_widget('checkbutton_use_power_fencing').get_active():
			self.wTree.get_widget('summary_fencing_primary').set_text("Network Power Switch")
			if self.wTree.get_widget('checkbutton_use_drac_fencing').get_active():
				self.wTree.get_widget('summary_fencing_secondary').set_text("DRAC")
			elif self.wTree.get_widget('checkbutton_use_ipmi_fencing').get_active():
				self.wTree.get_widget('summary_fencing_secondary').set_text("IPMI")
			else:
				self.wTree.get_widget('summary_fencing_secondary').set_text("none")
		elif self.wTree.get_widget('checkbutton_use_drac_fencing').get_active():
			self.wTree.get_widget('summary_fencing_primary').set_text("DRAC")
			self.wTree.get_widget('summary_fencing_secondary').set_text("none")
		elif self.wTree.get_widget('checkbutton_use_ipmi_fencing').get_active():
			self.wTree.get_widget('summary_fencing_primary').set_text("IPMI")
			self.wTree.get_widget('summary_fencing_secondary').set_text("none")
		else:
			self.wTree.get_widget('summary_fencing_primary').set_text("none - DANGER: Fencing is required for data integrity!")
			self.wTree.get_widget('summary_fencing_secondary').set_text("none")
		if self.DEBUG: 
			repr( {"checkbutton_use_power_fencing" : self.wTree.get_widget('checkbutton_use_power_fencing').get_active(),
				"checkbutton_use_drac_fencing" : self.wTree.get_widget('checkbutton_use_drac_fencing').get_active(),
				"checkbutton_use_ipmi_fencing" : self.wTree.get_widget('checkbutton_use_ipmi_fencing').get_active() })
			print "DEBUG: Dumping cluster information:"
			print "  clustername value:", self.wTree.get_widget('cluster_name').get_text()
			print "  DRAC/IPMI netmask: ", self.wTree.get_widget('entry_drac_netmask').get_text()
			print "  DRAC/IPMI gateway: ", self.wTree.get_widget('entry_drac_gateway').get_text()
			print "  cluster nodes:", self.wTree.get_widget('cluster_nodes').get_active_text()
		if self.wTree.get_widget('radiobutton_md3000').get_active():
			self.wTree.get_widget('summary_cluster_storage_type').set_text("MD3000 SAS")
			self.wTree.get_widget('label_summary_cluster_storage_iscsi').hide()
			self.wTree.get_widget('summary_cluster_storage_iscsi').hide()
		elif self.wTree.get_widget('radiobutton_md3000i').get_active():
			self.wTree.get_widget('label_summary_cluster_storage_iscsi').show()
			self.wTree.get_widget('summary_cluster_storage_iscsi').show()
			self.wTree.get_widget('summary_cluster_storage_type').set_text("MD3000i iSCSI")
			self.wTree.get_widget('summary_cluster_storage_iscsi').set_text(self.wTree.get_widget('entry_iscsi_target').get_text())
		elif self.wTree.get_widget('radiobutton_equallogic').get_active():
			self.wTree.get_widget('label_summary_cluster_storage_iscsi').show()
			self.wTree.get_widget('summary_cluster_storage_iscsi').show()
			self.wTree.get_widget('summary_cluster_storage_type').set_text("EqualLogic iSCSI")
			self.wTree.get_widget('summary_cluster_storage_iscsi').set_text(self.wTree.get_widget('entry_iscsi_target').get_text())

	def show_apps_info(self):
		if self.wTree.get_widget('radiobutton_md3000').get_active():
			self.wTree.get_widget('summary_storage_type').set_text("MD3000 SAS")
			self.wTree.get_widget('label_summary_storage_iscsi').hide()
			self.wTree.get_widget('summary_storage_iscsi').hide()
		elif self.wTree.get_widget('radiobutton_md3000i').get_active():
			self.wTree.get_widget('label_summary_storage_iscsi').show()
			self.wTree.get_widget('summary_storage_iscsi').show()
			self.wTree.get_widget('summary_storage_type').set_text("MD3000i iSCSI")
			self.wTree.get_widget('summary_storage_iscsi').set_text(self.wTree.get_widget('entry_iscsi_target').get_text())
		elif self.wTree.get_widget('radiobutton_equallogic').get_active():
			self.wTree.get_widget('label_summary_storage_iscsi').show()
			self.wTree.get_widget('summary_storage_iscsi').show()
			self.wTree.get_widget('summary_storage_type').set_text("EqualLogic iSCSI")
			self.wTree.get_widget('summary_storage_iscsi').set_text(self.wTree.get_widget('entry_iscsi_target').get_text())
		if self.wTree.get_widget('checkbutton_nfs').get_active():
			self.wTree.get_widget('summary_nfs').set_text("Export for " + self.wTree.get_widget('entry_nfs_ip').get_text() + ": \n" + self.wTree.get_widget('entry_nfs_mount').get_text() + "\t" + self.wTree.get_widget('entry_nfs_clients').get_text() + "(" + self.wTree.get_widget('entry_nfs_client_opts').get_text() + ")")
			if self.DEBUG: print "  ENabling NFS: ", \
				self.wTree.get_widget('entry_nfs_ip').get_text(), ",", \
				self.wTree.get_widget('entry_nfs_dev').get_text(), ",", \
				self.wTree.get_widget('entry_nfs_mount').get_text(), ",", \
				self.wTree.get_widget('entry_nfs_clients').get_text(), ",", \
				self.wTree.get_widget('entry_nfs_client_opts').get_text()
		else:
			self.wTree.get_widget('summary_nfs').set_text("not selected")
		if self.wTree.get_widget('checkbutton_web').get_active():
			self.wTree.get_widget('summary_web').set_text("http://" + self.wTree.get_widget('entry_web_ip').get_text())
			if self.DEBUG: print "  Enabling WEB: ",  \
				self.wTree.get_widget('entry_web_ip').get_text(), ",", \
				self.wTree.get_widget('entry_web_dev').get_text(), ",", \
				self.wTree.get_widget('entry_web_mount').get_text()
		else:
			self.wTree.get_widget('summary_web').set_text("not selected")
		if self.wTree.get_widget('checkbutton_samba').get_active():
			if self.DEBUG: print "  Enabling Samba: ",  \
				self.wTree.get_widget('entry_smb_ip').get_text(), ",", \
				self.wTree.get_widget('entry_smb_dev').get_text(), ",", \
				self.wTree.get_widget('entry_smb_mount').get_text(), ",", \
				self.wTree.get_widget('entry_smb_workgroup').get_text()
			self.wTree.get_widget('summary_samba').set_text("smb://" + self.wTree.get_widget('entry_smb_ip').get_text() + "/pub" + "\nWorkgroup: " + self.wTree.get_widget('entry_smb_workgroup').get_text())
		else:
			self.wTree.get_widget('summary_samba').set_text("not selected")
		if self.wTree.get_widget('checkbutton_virt').get_active():
			if self.DEBUG: print "  Enabling Virt Guest: ",  \
				self.wTree.get_widget('entry_virt_dev').get_text(), ",", \
				self.wTree.get_widget('entry_virt_mount').get_text()
			self.wTree.get_widget('summary_virt').set_text("configure")
		else:
			self.wTree.get_widget('summary_virt').set_text("not selected")
		if self.wTree.get_widget('checkbutton_ftp').get_active():
			if self.DEBUG: print "  Enabling FTP: ",  \
				self.wTree.get_widget('entry_ftp_ip').get_text(), ",", \
				self.wTree.get_widget('entry_ftp_dev').get_text(), ",", \
				self.wTree.get_widget('entry_ftp_mount').get_text()
			self.wTree.get_widget('summary_ftp').set_text("ftp://" + self.wTree.get_widget('entry_ftp_ip').get_text())
		else:
			self.wTree.get_widget('summary_ftp').set_text("not selected")
		# Set device in summary
		self.wTree.get_widget('summary_storage_dev').set_text(self.wTree.get_widget('entry_storage_dev').get_text())
		self.wTree.get_widget('summary_storage_mount_point').set_text(self.wTree.get_widget('entry_storage_mount_point').get_text())

	def role_chooser(self, checkbutton):
		if self.wTree.get_widget('checkbutton_mgmt').get_active():
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_ROLE_CHOOSER), True)
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_START), True)
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_INFO), True)
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APPS), True)
		elif self.wTree.get_widget('checkbutton_cluster').get_active():
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_ROLE_CHOOSER), True)
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CLUSTER_START), True)
		else:
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_ROLE_CHOOSER), False)

	def reconfigure_toggled(self, checkbutton):
		if self.DEBUG: print "reconfigure button state:", checkbutton.get_active()
		if checkbutton.get_active():
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_RECONFIGURE), True)

	def validate_mount_point_input(self, input):
		if input.get_text() and input.get_text().rstrip(' /'):
			if self.valid_mount_point(input.get_text()) and self.valid_mount_point(input.get_text().rstrip(' /')):
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CONFIRM_STORAGE), True)
			else:
				if self.DEBUG: print "Mount point: is not valid"
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CONFIRM_STORAGE), False)
		else:
			if self.DEBUG: print "Mount point: is not valid"
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_CONFIRM_STORAGE), False)

	def validate_nfs_input(self, input):
		if not input.get_text():
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_NFS), False)
			return False
		if not self.valid_ip(self.wTree.get_widget('entry_nfs_ip').get_text()) or not self.validate_nfs_client(self.wTree.get_widget('entry_nfs_clients').get_text()):
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_NFS), False)
			return False
		else:
			self.wTree.get_widget('entry_nfs_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
			self.wTree.get_widget('entry_nfs_clients').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
			self.message_id = self.wTree.get_widget('statusbar_nfs').push(self.nfs_context_id, "")
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_NFS), True)
			return True
		
	def validate_nfs_field_handler(self, input, event):
		self.validate_nfs_field(input, event)
		return False

	def validate_nfs_field(self, input, event):
		if self.DEBUG: print "running NFS validation"
		if self.wTree.get_widget('entry_nfs_ip').get_text():
			if self.DEBUG: print "running NFS validation on ip"
			if self.valid_ip(self.wTree.get_widget('entry_nfs_ip').get_text()):
				self.wTree.get_widget('entry_nfs_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				self.message_id = self.wTree.get_widget('statusbar_nfs').push(self.nfs_context_id, "")
			else:
				# color red and write to status bar
				if self.DEBUG: print "invalid ip for nfs ip entry"
				self.wTree.get_widget('entry_nfs_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
				self.message_id = self.wTree.get_widget('statusbar_nfs').push(self.nfs_context_id, "Invalid IP Address!")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_NFS), False)
				return False
		else:
			# One of the fields is empty, let's not yell at them yet
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_NFS), False)
			return False
		if self.wTree.get_widget('entry_nfs_clients').get_text():
			if self.DEBUG: print "running NFS validation on clients"
			if self.validate_nfs_client(self.wTree.get_widget('entry_nfs_clients').get_text()):
				self.wTree.get_widget('entry_nfs_clients').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				self.message_id = self.wTree.get_widget('statusbar_nfs').push(self.nfs_context_id, "")
			else:
				# color red and write to status bar
				if self.DEBUG: print "invalid client specification for nfs ip entry"
				self.wTree.get_widget('entry_nfs_clients').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
				self.message_id = self.wTree.get_widget('statusbar_nfs').push(self.nfs_context_id, "Invalid client entry!")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_NFS), False)
				return False
		else:
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_NFS), False)
			return False
		if self.valid_ip(self.wTree.get_widget('entry_nfs_ip').get_text()) and self.validate_nfs_client(self.wTree.get_widget('entry_nfs_clients').get_text()):
			if self.DEBUG: print "Everything passed NFS validation"
			self.message_id = self.wTree.get_widget('statusbar_nfs').push(self.nfs_context_id, "")
			self.wTree.get_widget('entry_nfs_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
			self.wTree.get_widget('entry_nfs_clients').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_NFS), True)
			return True
		else:
			return False
		
	def validate_web_input(self, input):
		if not input.get_text():
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_WEB), False)
			return False
		if not self.valid_ip(self.wTree.get_widget('entry_web_ip').get_text()):
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_WEB), False)
			return False
		else:
			self.wTree.get_widget('entry_web_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
			self.message_id = self.wTree.get_widget('statusbar_web').push(self.web_context_id, "")
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_WEB), True)
			return True
		
	def validate_web_field_handler(self, input, event):
		self.validate_web_field(input, event)
		return False

	def validate_web_field(self, input, event):
		if self.wTree.get_widget('entry_web_ip').get_text():
			if self.valid_ip(self.wTree.get_widget('entry_web_ip').get_text()):
				self.wTree.get_widget('entry_web_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				self.message_id = self.wTree.get_widget('statusbar_web').push(self.web_context_id, "")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_WEB), True)
				return True
			else:
				# color red and write to status bar
				if self.DEBUG: print "invalid ip for web ip entry"
				self.wTree.get_widget('entry_web_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
				self.message_id = self.wTree.get_widget('statusbar_web').push(self.web_context_id, "Invalid IP Address!")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_WEB), False)
				return False
		else:
			# One of the fields is empty, let's not yell at them yet
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_WEB), False)
			return False
		
	def validate_smb_input(self, input):
		if not input.get_text():
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_SAMBA), False)
			return False
		if not self.valid_ip(self.wTree.get_widget('entry_smb_ip').get_text()):
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_SAMBA), False)
			return False
		else:
			if self.validate_smb_field(input, None):
				self.wTree.get_widget('entry_smb_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				self.wTree.get_widget('entry_smb_workgroup').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				self.message_id = self.wTree.get_widget('statusbar_smb').push(self.smb_context_id, "")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_SAMBA), True)
				return True
			else:
				return False
		
	def validate_smb_field_handler(self, input, event):
		self.validate_smb_field(input, event)
		return False

	def validate_smb_field(self, input, event):
		if self.DEBUG: print "checking" + repr(input.get_text())
		if self.wTree.get_widget('entry_smb_ip').get_text():
			if self.valid_ip(self.wTree.get_widget('entry_smb_ip').get_text()):
				self.wTree.get_widget('entry_smb_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				self.message_id = self.wTree.get_widget('statusbar_smb').push(self.smb_context_id, "")
			else:
				# color red and write to status bar
				if self.DEBUG: print "invalid ip for smb ip entry"
				self.wTree.get_widget('entry_smb_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
				self.message_id = self.wTree.get_widget('statusbar_smb').push(self.smb_context_id, "Invalid IP Address!")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_SAMBA), False)
				return False
		else:
			# One of the fields is empty, let's not yell at them yet
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_SAMBA), False)
			return False
		if self.wTree.get_widget('entry_smb_workgroup').get_text():
			if self.valid_workgroup(self.wTree.get_widget('entry_smb_workgroup').get_text()):
				self.wTree.get_widget('entry_smb_workgroup').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				self.message_id = self.wTree.get_widget('statusbar_smb').push(self.smb_context_id, "")
			else:
				# color red and write to status bar
				if self.DEBUG: print "invalid workgroup entry"
				self.wTree.get_widget('entry_smb_workgroup').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
				self.message_id = self.wTree.get_widget('statusbar_smb').push(self.smb_context_id, "Invalid workgroup entry!")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_SAMBA), False)
				return False
			if self.valid_workgroup_len(self.wTree.get_widget('entry_smb_workgroup').get_text()):
				self.wTree.get_widget('entry_smb_workgroup').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				self.message_id = self.wTree.get_widget('statusbar_smb').push(self.smb_context_id, "")
			else:
				# color red and write to status bar
				if self.DEBUG: print "invalid workgroup length"
				self.wTree.get_widget('entry_smb_workgroup').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
				self.message_id = self.wTree.get_widget('statusbar_smb').push(self.smb_context_id, "Invalid workgroup length! Workgroup must be 15 characters or less.")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_SAMBA), False)
				return False
		else:
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_SAMBA), False)
			return False
		if self.valid_ip(self.wTree.get_widget('entry_smb_ip').get_text()) and self.valid_workgroup(self.wTree.get_widget('entry_smb_workgroup').get_text()) and self.valid_workgroup_len(self.wTree.get_widget('entry_smb_workgroup').get_text()):
			self.message_id = self.wTree.get_widget('statusbar_smb').push(self.smb_context_id, "")
			self.wTree.get_widget('entry_smb_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
			self.wTree.get_widget('entry_smb_workgroup').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_SAMBA), True)
		return False
			
	def validate_virt_input(self, input):
		if not input.get_text():
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_VIRT), False)
			return False
		if not self.valid_ip(self.wTree.get_widget('entry_virt_ip').get_text()):
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_VIRT), False)
			return False
		else:
			self.wTree.get_widget('entry_virt_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
			self.message_id = self.wTree.get_widget('statusbar_virt').push(self.virt_context_id, "")
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_VIRT), True)
			return True
		
	def validate_virt_field_handler(self, input, event):
		self.validate_virt_field(input, event)
		return False

	def validate_virt_field(self, input, event):
		if self.wTree.get_widget('entry_virt_ip').get_text():
			if self.valid_ip(self.wTree.get_widget('entry_virt_ip').get_text()):
				self.wTree.get_widget('entry_virt_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				self.message_id = self.wTree.get_widget('statusbar_virt').push(self.virt_context_id, "")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_VIRT), True)
			else:
				# color red and write to status bar
				if self.DEBUG: print "invalid ip for virt ip entry"
				self.wTree.get_widget('entry_virt_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
				self.message_id = self.wTree.get_widget('statusbar_virt').push(self.virt_context_id, "Invalid IP Address!")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_VIRT), False)
				return False
		else:
			# One of the fields is empty, let's not yell at them yet
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_VIRT), False)
			return False
			
	def validate_ftp_input(self, input):
		if not input.get_text():
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_FTP), False)
			return False
		if not self.valid_ip(self.wTree.get_widget('entry_ftp_ip').get_text()):
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_FTP), False)
			return False
		else:
			self.wTree.get_widget('entry_ftp_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
			self.message_id = self.wTree.get_widget('statusbar_ftp').push(self.ftp_context_id, "")
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_FTP), True)
			return True
		
	def validate_ftp_field_handler(self, input, event):
		self.validate_ftp_field(input, event)
		return False

	def validate_ftp_field(self, input, event):
		if self.wTree.get_widget('entry_ftp_ip').get_text():
			if self.valid_ip(self.wTree.get_widget('entry_ftp_ip').get_text()):
				self.wTree.get_widget('entry_ftp_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
				self.message_id = self.wTree.get_widget('statusbar_ftp').push(self.ftp_context_id, "")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_FTP), True)
				return True
			else:
				# color red and write to status bar
				if self.DEBUG: print "invalid ip for ftp ip entry"
				self.wTree.get_widget('entry_ftp_ip').modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("red"))
				self.message_id = self.wTree.get_widget('statusbar_ftp').push(self.ftp_context_id, "Invalid IP Address!")
				self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_FTP), False)
				return False
		else:
			# One of the fields is empty, let's not yell at them yet
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_APP_FTP), False)
			return False
	
	def validate_iscsi_input(self, input):
		if not self.wTree.get_widget('entry_iscsi_target').get_text():
			if self.DEBUG: print "iSCSI field is still not valid"
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_STORAGE_ISCSI), False)
		else:
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_STORAGE_ISCSI), True)

	def checkbutton_storage_complete_toggled(self, state):
		if state.get_active():
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_STORAGE_CONFIG), True)
		else:
			self.dccg.set_page_complete(self.dccg.get_nth_page(self.PAGE_STORAGE_CONFIG), False)

	def launch_url(self, url, event):
		# Normally the webbrowser module works great.  However, for Firefox3 there is an
		# open bug that causes an absolute path to be prepended.  Bug: http://bugs.python.org/issue1911
		# So forcing webbrowser to open using the GenericBrowser function if firefox is present, otherwise
		# use the normal webbrowser.open module
		browser = False
		browser = webbrowser.get('firefox')
		if browser:
			if self.DEBUG: print "Found firefox, launching: ", url.get_text()
			browser.open(url.get_text(),0,1)
		else:
			if self.DEBUG: print "Launching: ", url.get_text()
			webbrowser.open(url.get_text(),0,1)
		
	def valid_ip(self, ip_str):
		if len(ip_str.split()) == 1:
			ipList = ip_str.split('.')
			if len(ipList) == 4:
				for i, item in enumerate(ipList):
					try:
						ipList[i] = int(item)
					except:
						return False
					if not isinstance(ipList[i], int):
						return False
				if max(ipList) < 256:
					return True
				else:
					return False
			else:
				return False
		else:
			return False

if __name__ == "__main__":
	gtk.gdk.threads_init()
	hwg = DellConfigClusterGui()
	gtk.main()
